@@ -166,10 +166,11 @@ struct NautilusIconContainerDetails {
 	/* zoom level */
 	int zoom_level;
 
-	/* fonts used to draw labels */
-	char *font_name;
+	/* specific fonts used to draw labels */
+	char *font;
+	
+	/* font sizes used to draw labels */
 	int font_size_table[NAUTILUS_ZOOM_LEVEL_LARGEST + 1];
-	gboolean update_icon_font;
 
 	/* pixbuf and color for label highlighting */
 	GdkPixbuf *highlight_frame;
@@ -206,8 +207,8 @@ struct NautilusIconContainerDetails {
 	gboolean reset_scroll_region_trigger;
 	
 	/* The position we are scaling to on stretch */
-	int widget_x;
-	int widget_y;
+	double world_x;
+	double world_y;
 
 	/* margins to follow, used for the desktop panel avoidance */
 	int left_margin;
