@@ -190,41 +190,6 @@ compute_title (NautilusWindow *window)
         return title;
 }
 
-/* window_set_title_with_time_stamp
- * 
- * Update the non-NautilusViewFrame objects that use the location's user-displayable
- * title in some way. Called when the location or title has changed.
- * @window: The NautilusWindow in question.
- * @title: The new user-displayable title.
- * 
- */
-static void
-window_set_title_with_time_stamp (NautilusWindow *window, const char *title)
-{
-        char *time_stamp;
-	char *title_with_time_stamp;
-	
-        g_return_if_fail (NAUTILUS_IS_WINDOW (window));
-        g_return_if_fail (title != NULL);
-
-	time_stamp = nautilus_get_build_time_stamp ();
-	
-	if (time_stamp != NULL) {
-		/* FIXME bugzilla.gnome.org 45037: The text Preview
-		 * Release is hardcoded here. Are all builds with
-		 * time stamps really best described as "preview
-		 * release"?.
-                 */
-		title_with_time_stamp = g_strdup_printf (_("Preview Release %s: %s"), time_stamp, title);
-		gtk_window_set_title (GTK_WINDOW (window), title_with_time_stamp);
-		g_free (title_with_time_stamp);
-	} else {
-		gtk_window_set_title (GTK_WINDOW (window), title);
-	}
-
-	g_free (time_stamp);
-}
-
 /* update_title:
  * 
  * Update the non-NautilusViewFrame objects that use the location's user-displayable
@@ -253,10 +218,10 @@ update_title (NautilusWindow *window)
         window->details->title = g_strdup (title);
 
         if (title[0] == '\0') {
-		window_set_title_with_time_stamp (window, _("Nautilus"));
+		gtk_window_set_title (GTK_WINDOW (window), _("Nautilus"));
         } else {
                 window_title = eel_str_middle_truncate (title, MAX_TITLE_LENGTH);
-		window_set_title_with_time_stamp (window, window_title);
+                gtk_window_set_title (GTK_WINDOW (window), window_title);
                 g_free (window_title);
         }
 
@@ -536,10 +501,14 @@ viewed_file_changed_callback (NautilusFile *file,
 static void
 cancel_viewed_file_changed_callback (NautilusWindow *window)
 {
-        if (window->details->viewed_file != NULL) {
-                g_signal_handlers_disconnect_by_func (window->details->viewed_file,
+        NautilusFile *file;
+
+        file = window->details->viewed_file;
+        if (file != NULL) {
+                g_signal_handlers_disconnect_by_func (G_OBJECT (file),
                                                       G_CALLBACK (viewed_file_changed_callback),
                                                       window);
+                nautilus_file_monitor_remove (file, &window->details->viewed_file);
         }
 }
 
@@ -594,10 +563,9 @@ update_for_new_location (NautilusWindow *window)
         file = nautilus_file_get (window->details->location);
         nautilus_window_set_viewed_file (window, file);
         window->details->viewed_file_seen = !nautilus_file_is_not_yet_confirmed (file);
-        g_signal_connect (file,
-                            "changed",
-                            G_CALLBACK (viewed_file_changed_callback),
-                            window);
+        nautilus_file_monitor_add (file, &window->details->viewed_file, NULL);
+        g_signal_connect_object (file, "changed",
+                                 G_CALLBACK (viewed_file_changed_callback), window, 0);
         nautilus_file_unref (file);
         
         /* Check if we can go up. */
@@ -640,6 +608,12 @@ ref_now_unref_at_idle_time (GObject *object)
         g_idle_add (unref_callback, object);
 }
 
+static gboolean
+use_saved_window_positions (void)
+{
+        return eel_preferences_get_boolean (NAUTILUS_PREFERENCES_WINDOW_ALWAYS_NEW);
+}
+
 /* This is called when we have decided we can actually change to the new view/location situation. */
 static void
 location_has_really_changed (NautilusWindow *window)
@@ -678,12 +652,18 @@ location_has_really_changed (NautilusWindow *window)
         free_location_change (window);
 
         update_title (window);
-}
 
-static gboolean
-use_saved_window_positions (void)
-{
-        return eel_preferences_get_boolean (NAUTILUS_PREFERENCES_WINDOW_ALWAYS_NEW);
+        /* The whole window has been finished. Now show it, unless
+         * we're still waiting for the saved positions from the
+         * metadata. Then tell the callback it needs to show the
+         * window
+         */
+        if (!use_saved_window_positions () ||
+            window->show_state == NAUTILUS_WINDOW_POSITION_SET) {
+                gtk_widget_show (GTK_WIDGET (window));
+        } else {
+                window->show_state = NAUTILUS_WINDOW_SHOULD_SHOW;
+        }
 }
 
 static void
@@ -719,9 +699,6 @@ open_location (NautilusWindow *window,
 
         if (create_new_window) {
                 target_window = nautilus_application_create_window (window->application);
-                if (!use_saved_window_positions ()) {
-                        gtk_widget_show (GTK_WIDGET (target_window));
-                }
         }
 
 	eel_g_list_free_deep (target_window->details->pending_selection);
@@ -1038,11 +1015,11 @@ report_sidebar_panel_failure_to_user (NautilusWindow *window, NautilusViewFrame
 static void
 disconnect_and_destroy_sidebar_panel (NautilusWindow *window, NautilusViewFrame *view)
 {
-        gtk_widget_ref (GTK_WIDGET (view));
+        g_object_ref (view);
 	disconnect_view (window, view);
         nautilus_window_remove_sidebar_panel (window, view);
-	gtk_widget_destroy (GTK_WIDGET (view));
-        gtk_widget_unref (GTK_WIDGET (view));
+	gtk_object_destroy (GTK_OBJECT (view));
+        g_object_unref (view);
 }
 
 static void
@@ -1156,8 +1133,14 @@ position_and_show_window_callback (NautilusFile *file,
 		g_free (geometry_string);
 	}
 
-	gtk_widget_show (GTK_WIDGET (window));
-
+        /* If we finished constructing the window by now we need
+         * to show the window here.
+         */
+        if (window->show_state == NAUTILUS_WINDOW_SHOULD_SHOW) {
+                gtk_widget_show (GTK_WIDGET (window));
+        }
+        window->show_state = NAUTILUS_WINDOW_POSITION_SET;
+        
         /* This object was ref'd when starting the callback. */
         nautilus_file_unref (file);
 }                       			     
@@ -1203,9 +1186,8 @@ determined_initial_view_callback (NautilusDetermineViewHandle *handle,
 		 * windows), position and show it only after we've got the
 		 * metadata (since position info is stored there).
 		 */
-                if (!use_saved_window_positions ()) {
-                        gtk_widget_show (GTK_WIDGET (window));
-                } else {
+                if (use_saved_window_positions ()) {
+                        window->show_state = NAUTILUS_WINDOW_NOT_SHOWN;
                         if (!GTK_WIDGET_VISIBLE (window)) {
                                 file = nautilus_file_get (location);
                                 
@@ -1417,6 +1399,8 @@ begin_location_change (NautilusWindow *window,
                   || type == NAUTILUS_LOCATION_CHANGE_FORWARD
                   || distance == 0);
 
+        g_object_ref (window);
+
         end_location_change (window);
         
         nautilus_window_allow_stop (window, TRUE);
@@ -1449,9 +1433,9 @@ begin_location_change (NautilusWindow *window,
         nautilus_directory_unref (directory);
         
         window->details->determine_view_handle = nautilus_determine_initial_view
-                (location,
-                 determined_initial_view_callback,
-                 window);
+                (location, determined_initial_view_callback, window);
+
+        g_object_unref (window);
 }
 
 static void
@@ -1620,7 +1604,7 @@ static void
 zoom_parameters_changed_callback (NautilusViewFrame *view,
                                   NautilusWindow *window)
 {
-        double zoom_level;
+        float zoom_level;
 
         g_assert (NAUTILUS_IS_WINDOW (window));
 
@@ -1694,8 +1678,7 @@ get_history_list_callback (NautilusViewFrame *view,
 	NautilusBookmark *bookmark;
 	int length, i;
 	GList *node;
-	char *name, *location, *pixbuf_xml;
-	GdkPixbuf *pixbuf;
+	char *name, *location;
 	
 	/* Get total number of history items */
 	length = g_list_length (nautilus_get_history_list ());
@@ -1706,24 +1689,19 @@ get_history_list_callback (NautilusViewFrame *view,
 	list->_maximum = length;
 	list->_buffer = CORBA_sequence_Nautilus_HistoryItem_allocbuf (length);
 	CORBA_sequence_set_release (list, CORBA_TRUE);
-	
+
 	/* Iterate through list and copy item data */
 	for (i = 0, node = nautilus_get_history_list (); i < length; i++, node = node->next) {
 		bookmark = node->data;
 
 		name = nautilus_bookmark_get_name (bookmark);
 		location = nautilus_bookmark_get_uri (bookmark);
-		pixbuf = nautilus_bookmark_get_pixbuf (bookmark, NAUTILUS_ICON_SIZE_FOR_MENUS, FALSE);
-		pixbuf_xml = bonobo_ui_util_pixbuf_to_xml (pixbuf);
 		
 		list->_buffer[i].title = CORBA_string_dup (name);
 		list->_buffer[i].location = CORBA_string_dup (location);
-		list->_buffer[i].icon = CORBA_string_dup (pixbuf_xml);
-
+                
 		g_free (name);
 		g_free (location);
-		g_free (pixbuf_xml);
-		g_object_unref (pixbuf);		
 	}
 
 	return list;
@@ -2024,15 +2002,17 @@ disconnect_view_callback (gpointer list_item_data, gpointer callback_data)
 void
 nautilus_window_manage_views_destroy (NautilusWindow *window)
 {
-        free_location_change (window);
-
 	/* Disconnect view signals here so they don't trigger when
 	 * views are destroyed.
          */
 	g_list_foreach (window->sidebar_panels, disconnect_view_callback, window);
         disconnect_view (window, window->content_view);
+}
 
-        /* Cancel callbacks. */
+void
+nautilus_window_manage_views_finalize (NautilusWindow *window)
+{
+        free_location_change (window);
         cancel_viewed_file_changed_callback (window);
 }
 
@@ -2041,9 +2021,20 @@ nautilus_window_back_or_forward (NautilusWindow *window, gboolean back, guint di
 {
 	GList *list;
 	char *uri;
+        guint len;
 	
 	list = back ? window->back_list : window->forward_list;
-	g_assert (g_list_length (list) > distance);
+
+        len = (guint) g_list_length (list);
+
+        /* If we can't move in the direction at all, just return. */
+        if (len == 0)
+                return;
+
+        /* If the distance to move is off the end of the list, go to the end
+           of the list. */
+        if (distance >= len)
+                distance = len - 1;
 
 	uri = nautilus_bookmark_get_uri (g_list_nth_data (list, distance));
 	begin_location_change
