@@ -355,6 +355,7 @@ back_or_forward_toolbar_item_property_set_cb (BonoboPropertyBag *bag,
 static BonoboUIToolbarButtonItem *
 create_back_or_forward_toolbar_item (NautilusWindow *window, 
 				     const char     *label, 
+				     const char     *tooltip,
 				     const char     *control_path)
 {
 	BonoboUIToolbarButtonItem *item;
@@ -367,6 +368,9 @@ create_back_or_forward_toolbar_item (NautilusWindow *window,
 	gtk_widget_show (GTK_WIDGET (item));
 
 	button = bonobo_ui_toolbar_button_item_get_button_widget (item);
+	gtk_tooltips_set_tip (window->details->tooltips,
+			      GTK_WIDGET (button),
+			      tooltip, NULL);
 	g_signal_connect_object (button, "button_press_event",
 				 G_CALLBACK (back_or_forward_button_pressed_callback),
 				 window, 0);
@@ -439,6 +443,22 @@ throbber_callback (BonoboListener *listener,
 	}
 }
 
+static void
+throbber_set_throbbing (NautilusWindow *window,
+			gboolean        throbbing)
+{
+	CORBA_boolean b;
+	CORBA_any     val;
+
+	val._type = TC_CORBA_boolean;
+	val._value = &b;
+	b = throbbing;
+
+	eel_bonobo_pbclient_set_value_async (
+		window->details->throbber_property_bag,
+		"throbbing", &val, NULL);
+}
+
 static void
 throbber_created_callback (Bonobo_Unknown     throbber,
 			   CORBA_Environment *ev,
@@ -458,6 +478,8 @@ throbber_created_callback (Bonobo_Unknown     throbber,
 
 	window = NAUTILUS_WINDOW (user_data);
 
+	window->details->throbber_activating = FALSE;
+
 	bonobo_ui_component_object_set (window->details->shell_ui,
 					"/Toolbar/ThrobberWrapper",
 					throbber, ev);
@@ -470,10 +492,7 @@ throbber_created_callback (Bonobo_Unknown     throbber,
 		window->details->throbber_property_bag = CORBA_OBJECT_NIL;
 		CORBA_exception_free (ev);
 	} else {
-		bonobo_pbclient_set_boolean (window->details->throbber_property_bag,
-					     "throbbing",
-					     window->details->throbber_active,
-					     ev);
+		throbber_set_throbbing (window, window->details->throbber_active);
 	}
 
 	window->details->throbber_listener =
@@ -490,8 +509,6 @@ throbber_created_callback (Bonobo_Unknown     throbber,
 void
 nautilus_window_allow_stop (NautilusWindow *window, gboolean allow)
 {
-	CORBA_Environment ev;
-	
 	if (( window->details->throbber_active &&  allow) ||
 	    (!window->details->throbber_active && !allow)) {
 		return;
@@ -510,22 +527,23 @@ nautilus_window_allow_stop (NautilusWindow *window, gboolean allow)
 				       NAUTILUS_COMMAND_STOP, allow);
 
 	if (window->details->throbber_property_bag != CORBA_OBJECT_NIL) {
-		CORBA_exception_init (&ev);
-		bonobo_pbclient_set_boolean (window->details->throbber_property_bag,
-					     "throbbing", allow, &ev);
-		CORBA_exception_free (&ev);
+		throbber_set_throbbing (window, allow);
 	}
 
 	nautilus_window_ui_thaw (window);
 }
 
+
 void
-nautilus_window_initialize_toolbars (NautilusWindow *window)
+nautilus_window_activate_throbber (NautilusWindow *window)
 {
 	CORBA_Environment ev;
 	char *exception_as_text;
 
-	nautilus_window_ui_freeze (window);
+	if (window->details->throbber_activating ||
+	    window->details->throbber_property_bag != CORBA_OBJECT_NIL) {
+		return;
+	}
 
 	/* FIXME bugzilla.gnome.org 41243: 
 	 * We should use inheritance instead of these special cases
@@ -535,7 +553,7 @@ nautilus_window_initialize_toolbars (NautilusWindow *window)
 		CORBA_exception_init (&ev);
 
 		g_object_ref (window);
-		bonobo_get_object_async ("OAFIID:nautilus_throbber",
+		bonobo_get_object_async ("OAFIID:Nautilus_Throbber",
 					 "IDL:Bonobo/Control:1.0",
 					 &ev,
 					 throbber_created_callback,
@@ -547,19 +565,31 @@ nautilus_window_initialize_toolbars (NautilusWindow *window)
 			g_free (exception_as_text);
 		}
 		CORBA_exception_free (&ev);
+		window->details->throbber_activating = TRUE;		
 	}
-	
+}
+
+void
+nautilus_window_initialize_toolbars (NautilusWindow *window)
+{
+	nautilus_window_ui_freeze (window);
+
+	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_START_WITH_TOOLBAR)) {
+		nautilus_window_activate_throbber (window);
+	}
+
 	window->details->back_button_item = create_back_or_forward_toolbar_item 
-		(window, _("Back"), "/Toolbar/BackWrapper");
+		(window, _("Back"), _("Go to the previous visited location"),
+		 "/Toolbar/BackWrapper");
 	window->details->forward_button_item = create_back_or_forward_toolbar_item 
-		(window, _("Forward"), "/Toolbar/ForwardWrapper");
+		(window, _("Forward"), _("Go to the next visited location"),
+		 "/Toolbar/ForwardWrapper");
 
 	set_up_toolbar_images (window);
 
-	eel_preferences_add_callback
-		(NAUTILUS_PREFERENCES_THEME, 
-		 theme_changed_callback,
-		 window);
+	eel_preferences_add_callback (NAUTILUS_PREFERENCES_THEME, 
+				      theme_changed_callback,
+				      window);
 
 	nautilus_window_ui_thaw (window);
 }
